@@ -81,7 +81,9 @@ dtiData <- function(gradient,imagefile,ddim,bvalue=NULL,xind=NULL,yind=NULL,zind
                  as.double(maxvalue),
                  PACKAGE="dti")$si
 #  this replaces the content off all voxel with elements <=0 or >maxvalue by 0
-     dim(si) <- dimsi
+  if(all(si==as.integer(si))) si <- as.integer(si)
+##  reduce memory requirements if possible
+  dim(si) <- dimsi
   level <- max(mins0value,level*mean(si[,,,s0ind][si[,,,s0ind]>0])) # set level to level*mean  of positive s_0 values
   ddim0 <- as.integer(ddim)
   ddim <- as.integer(dim(si)[1:3])
@@ -124,6 +126,7 @@ readDWIdata <- function(gradient, dirlist,
                         level = 0, mins0value = 1, maxvalue = 32000,
                         voxelext = NULL, orientation = c(0L, 2L, 5L), rotation = NULL,
                         pattern = NULL,
+                        SPM2 = TRUE,
                         verbose = FALSE) {
 
   args <- list(sys.call())
@@ -147,9 +150,15 @@ readDWIdata <- function(gradient, dirlist,
      if(length(bvalue)!=ngrad || max(bvalue[s0ind]) > 10*min(bvalue[-s0ind])) 
         stop("invalid b-values")
   }   
+  if(length(dirlist)==1&!is.na(isdir <- file.info(dirlist)$isdir)&!isdir){
+  ## dirlist contains a filename rather than a list of directories
+     filelist <- dirlist
+     dirlist <- NULL
+  } else {
   ## generate file list in specified order
-  filelist <- NULL
-  for (dd in dirlist) filelist <- c(filelist, list.files(dd, full.names = TRUE, pattern = pattern))
+     filelist <- NULL
+     for (dd in dirlist) filelist <- c(filelist, list.files(dd, full.names = TRUE, pattern = pattern))
+  }
   if ( length( filelist) == 0) stop( "readDWIdata: empty directories or directories do not exist!")
   if (format == "DICOM") {
     if (is.null(zind)) zind <- 1:nslice
@@ -206,7 +215,7 @@ readDWIdata <- function(gradient, dirlist,
     i <- i+1
     if (!verbose) setTxtProgressBar(pb, i)
     if (format == "DICOM") {
-      dd <- dicomInfo(ff)
+      dd <- readDICOM(ff)
       delta <- c(as.numeric(unlist(strsplit(extractHeader(dd$hdr, "PixelSpacing", FALSE)[1], " "))), extractHeader(dd$hdr, "SliceThickness")[1])
       imageOrientationPatient <- as.numeric(unlist(strsplit(extractHeader(dd$hdr, "ImageOrientationPatient", FALSE)[1], " ")))
       imageOrientationPatient <- matrix(c(imageOrientationPatient, vcrossp(imageOrientationPatient[1:3], imageOrientationPatient[4:6])), 3, 3)
@@ -226,6 +235,7 @@ readDWIdata <- function(gradient, dirlist,
       imageOrientationPatient <- t(matrix(c(dd@srow_x[1:3]/dd@pixdim[2:4], dd@srow_y[1:3]/dd@pixdim[2:4], dd@srow_z[1:3]/dd@pixdim[2:4]), 3, 3))
     } else if (format == "ANALYZE") {
       dd <- readANALYZE(ff)
+      if ( SPM2) dd@.Data <- dd@.Data * dd@funused1
       nslice <- dim(dd)[3]
       if (is.null(zind)) zind <- 1:nslice
       delta <- dd@pixdim[2:4]
@@ -281,6 +291,8 @@ readDWIdata <- function(gradient, dirlist,
       }
     } else {
       if (length(filelist) > 1) { # list of 3D files
+## for dti_leipzig data we got length(dim(dd))==4
+        if(length(dim(dd))==4&&dim(dd)[4]==1) dim(dd) <- dim(dd)[1:3]
         if (first) { 
           ttt <- dd[xind, yind, zind]
           nttt <- dim(ttt)
@@ -337,6 +349,8 @@ readDWIdata <- function(gradient, dirlist,
                  as.integer(dimsi[4]),
                  as.double(maxvalue),
                  PACKAGE = "dti")$si
+  if(all(si==as.integer(si))) si <- as.integer(si)
+  ##  reduce memory requirements if possible
   dim(si) <- dimsi
 
   ## set level to level*mean  of positive s_0 values
@@ -402,6 +416,20 @@ function(x){
     invisible(NULL)
 })
 
+setMethod( "print", "dkiTensor",
+           function( x) {
+             cat( "  Object of class", class(x), "\n")
+             cat( "  Generated by calls   :\n")
+             print( x@call)
+             cat( "  Dimension            :", paste( x@ddim, collapse = "x"), "\n")
+             ns0 <- length( x@s0ind)
+             cat( "  Number of S0 images  :", paste( ns0, collapse = "x"), "\n")
+             cat( "  Number of Gradients  :", paste( x@ngrad - ns0, collapse = "x"), "\n")
+             cat( "  Source-Filename      :", x@source, "\n")
+             cat( "  Slots                :\n")
+             print( slotNames(x))
+             invisible( NULL)
+           })
 
 setMethod("print", "dwiMixtensor",
 function(x){
@@ -449,6 +477,21 @@ function(x){
     invisible(NULL)
 })
 
+setMethod("print","dkiIndices",
+          function(x){
+            cat("  Object of class", class(x),"\n")
+            cat("  Generated by calls   :\n")
+            print(x@call)
+            cat("  Dimension            :", paste(x@ddim, collapse="x"), "\n")
+            ns0 <- length(x@s0ind)
+            cat("  Number of S0 images  :", paste(ns0, collapse="x"), "\n")
+            cat("  Number of Gradients  :", paste(x@ngrad-ns0, collapse="x"), "\n")
+            cat("  Source-Filename      :", x@source, "\n")
+            cat("  Slots                :\n")
+            print(slotNames(x))
+            invisible(NULL)
+          })
+
 setMethod("print","dwiFiber",
 function(x){
     cat("  Object of class", class(x),"\n")
@@ -495,6 +538,20 @@ function(object){
     invisible(NULL)
 })
 
+setMethod( "show", "dkiTensor",
+           function( object) {
+             cat( "  Object of class", class( object), "\n")
+             cat( "  Generated by calls   :\n")
+             print( object@call)
+             cat( "  Dimension            :", paste( object@ddim, collapse = "x"), "\n")
+             ns0 <- length( object@s0ind)
+             cat( "  Number of S0 images  :", paste( ns0, collapse = "x"), "\n")
+             cat( "  Number of Gradients  :", paste( object@ngrad - ns0, collapse = "x"), "\n")
+             cat( "  Source-Filename      :", object@source, "\n")
+             cat( "  Slots                :\n")
+             print( slotNames(object))
+             invisible( NULL)
+           })
 
 setMethod("show", "dwiMixtensor",
 function(object){
@@ -525,6 +582,22 @@ function(object){
     print(slotNames(object))
     invisible(NULL)
 })
+
+setMethod("show", "dkiIndices",
+          function(object){
+            cat("  Object of class", class(object),"\n")
+            cat("  Generated by calls   :\n")
+            print(object@call)
+            cat("  Dimension            :", paste(object@ddim, collapse="x"), "\n")
+            ns0 <- length(object@s0ind)
+            cat("  Number of S0 images  :", paste(ns0, collapse="x"), "\n")
+            cat("  Number of Gradients  :", paste(object@ngrad-ns0, collapse="x"), "\n")
+            cat("  Source-Filename      :", object@source, "\n")
+            cat("  Slots                :\n")
+            print(slotNames(object))
+            invisible(NULL)
+          })
+
 setMethod("show","dwiFiber",
 function(object){
     cat("  Object of class", class(object),"\n")
@@ -552,7 +625,7 @@ function(object, ...){
     ns0 <- length(object@s0ind)
     cat("  Number of S0 images   :", paste(ns0, collapse="x"), "\n")
     cat("  Number of Gradients   :", paste(object@ngrad-ns0, collapse="x"), "\n")
-    cat("  Voxel extensions      :", paste(object@voxelext, collapse="x"), "\n")
+    cat("  Voxel extensions      :", paste(signif(object@voxelext,3), collapse="x"), "\n")
     cat("  Index of S0-Images    :", paste(object@s0ind, collapse="x"), "\n")
     cat("  Quantiles of S0-values:","\n")
     print(signif(quantile(object@si[,,,object@s0ind],...),3))
@@ -572,7 +645,7 @@ function(object, ...){
     ns0 <- length(object@s0ind)
     cat("  Number of S0 images   :", paste(ns0, collapse="x"), "\n")
     cat("  Number of Gradients   :", paste(object@ngrad-ns0, collapse="x"), "\n")
-    cat("  Voxel extensions      :", paste(object@voxelext, collapse="x"), "\n")
+    cat("  Voxel extensions      :", paste(signif(object@voxelext,3), collapse="x"), "\n")
     cat("  Quantiles of S0-values:","\n")
     print(signif(quantile(object@th0,...),3))
     cat("  Mean S0-value         :", paste(z <- signif(mean(object@th0),3),collapse="x"), "\n")
@@ -587,6 +660,31 @@ function(object, ...){
     invisible(NULL)
 })
 
+setMethod( "summary", "dkiTensor",
+           function( object, ...) {
+             cat( "  Object of class", class( object), "\n")
+             cat( "  Generated by calls    :\n")
+             print( object@call)
+             cat( "  Source-Filename       :", object@source, "\n")
+             cat( "  Dimension             :", paste( object@ddim, collapse = "x"), "\n")
+             ngrad <- object@ngrad
+             ns0 <- length( object@s0ind)
+             cat( "  Number of S0 images   :", paste( ns0, collapse = "x"), "\n")
+             cat( "  Number of Gradients   :", paste( object@ngrad - ns0, collapse = "x"), "\n")
+             cat( "  Voxel extensions      :", paste( object@voxelext, collapse = "x"), "\n")
+             cat( "  Quantiles of S0-values:", "\n")
+             print( signif( quantile( object@th0, ...), 3))
+             cat( "  Mean S0-value         :", paste( z <- signif( mean( object@th0), 3), collapse = "x"), "\n")
+             cat( "  Voxel in mask         :", paste( sum( object@mask), collapse = "x"), "\n")
+             cat( "  Spatial smoothness    :", paste( signif( object@bw, 3), collapse = "x"), "\n")
+             cat( "  mean variance         :", paste( signif( mean( object@sigma[ object@mask]), 3), collapse = "x"), "\n")
+             penBIC <- log( ngrad - ns0) / ( ngrad - ns0) * 6
+             cat( "  BIC                   :", paste( signif( mean( log( object@sigma[ object@mask])) + penBIC, 3), collapse = "x"), "\n")
+             cat( "  hmax                  :", paste( object@hmax, collapse = "x"), "\n")
+             if( length( object@outlier) > 0) cat( "  Number of outliers    :", paste( length( object@outlier), collapse = "x"), "\n")
+             cat( "\n")
+             invisible( NULL)
+           })
 
 setMethod("summary", "dwiMixtensor",
 function(object, ...){
@@ -599,7 +697,7 @@ function(object, ...){
     ns0 <- length(object@s0ind)
     cat("  Number of S0 images   :", paste(ns0, collapse="x"), "\n")
     cat("  Number of Gradients   :", paste(object@ngrad-ns0, collapse="x"), "\n")
-    cat("  Voxel extensions      :", paste(object@voxelext, collapse="x"), "\n")
+    cat("  Voxel extensions      :", paste(signif(object@voxelext,3), collapse="x"), "\n")
     cat("  Quantiles of S0-values:","\n")
     print(signif(quantile(object@th0,...),3))
     cat("  Mean S0-value         :", paste(z <- signif(mean(object@th0),3),collapse="x"), "\n")
@@ -626,7 +724,7 @@ function(object, ...){
     ns0 <- length(object@s0ind)
     cat("  Number of S0 images   :", paste(ns0, collapse="x"), "\n")
     cat("  Number of Gradients   :", paste(object@ngrad-ns0, collapse="x"), "\n")
-    cat("  Voxel extensions      :", paste(object@voxelext, collapse="x"), "\n")
+    cat("  Voxel extensions      :", paste(signif(object@voxelext,3), collapse="x"), "\n")
     cat("  Kind                  :", paste(object@what, collapse="x"), "\n")
     ord <- object@order
     cat("  Order                 :", paste(ord, collapse="x"), "\n")
@@ -653,7 +751,7 @@ function(object, ...){
     ns0 <- length(object@s0ind)
     cat("  Number of S0 images   :", paste(ns0, collapse="x"), "\n")
     cat("  Number of Gradients   :", paste(object@ngrad-ns0, collapse="x"), "\n")
-    cat("  Voxel extensions      :", paste(object@voxelext, collapse="x"), "\n")
+    cat("  Voxel extensions      :", paste(signif(object@voxelext,3), collapse="x"), "\n")
     cat("  Percentage of zero values      :",paste(signif(mean(object@fa==0)*100,3), "%",collapse="x"), "\n")
     cat("  Quantiles of positive FA-values:","\n")
     print(signif(quantile(object@fa[object@fa>0],...),3))
@@ -664,6 +762,29 @@ function(object, ...){
     cat("\n")
     invisible(NULL)
 })
+
+setMethod("summary", "dkiIndices",
+          function(object, ...){
+            cat("  Object of class", class(object),"\n")
+            cat("  Generated by calls    :\n")
+            print(object@call)
+            cat("  Source-Filename       :", object@source, "\n")
+            cat("  Dimension             :", paste(object@ddim, collapse="x"), "\n")
+            ns0 <- length(object@s0ind)
+            cat("  Number of S0 images   :", paste(ns0, collapse="x"), "\n")
+            cat("  Number of Gradients   :", paste(object@ngrad-ns0, collapse="x"), "\n")
+            cat("  Voxel extensions      :", paste(signif(object@voxelext,3), collapse="x"), "\n")
+            cat("  Percentage of zero values      :",paste(signif(mean(object@fa==0)*100,3), "%",collapse="x"), "\n")
+            cat("  Quantiles of positive FA-values:","\n")
+            print(signif(quantile(object@fa[object@fa>0],...),3))
+            cat("  Quantiles of positive GA-values:","\n")
+            print(signif(quantile(object@ga[object@ga>0],...),3))
+            cat("  Quantiles of positive MD-values:","\n")
+            print(signif(quantile(object@md[object@md>0],...),3))
+            cat("\n")
+            invisible(NULL)
+          })
+
 setMethod("summary","dwiFiber",
 function(object){
     cat("  Object of class", class(object),"\n")
@@ -674,7 +795,7 @@ function(object){
     ns0 <- length(object@s0ind)
     cat("  Number of S0 images   :", paste(ns0, collapse="x"), "\n")
     cat("  Number of Gradients   :", paste(object@ngrad-ns0, collapse="x"), "\n")
-    cat("  Voxel extensions      :", paste(object@voxelext, collapse="x"), "\n")
+    cat("  Voxel extensions      :", paste(signif(object@voxelext,3), collapse="x"), "\n")
     cat("  Minimum FA  :", object@minfa, "\n")
     cat("  Maximum angle :", object@maxangle , "\n")
     cat("  Number of fibers :", length(object@startind), "\n")
